
package com.zerog.ia.customcode.util.fileutils;

import java.io.*;

import com.zerog.ia.api.pub.*;

/**
 *	CopyDirectory takes a specified file and copies it
 *	to a new location.
 *
 *	@see com.acme.dialogs.CustomCodeAction
 *
 *	@version 3.0.0
 */
public class CopyFile extends CustomCodeAction
{
	private static final String INSTALL_MESSAGE = "Copying files";
	private static final String UNINSTALL_MESSAGE = "Deleting copied files";
	private static final String ERR_MSG =
		"CopyFile: no source or destination specified.";
	private static final String SOURCE_VAR_NAME = "$CopyFile_Source$";
	private static final String DEST_VAR_NAME = "$CopyFile_Destination$";
	private boolean isLoaded = false;
	/**
	 *	This is the method that is called at install-time. The InstallerProxy
	 *	instance provides methods to access information in the installer,
	 *	set status, and control flow.<p>
	 *
	 *	For the purposes of the this action (CopyFile), this method
	 *	<ol>
	 *	<li>gets its parameters from InstallAnywhere Variables,</li>
	 *	<li>checks the parameters' validity,</li>
	 *	<li>and copies the file.</li></ol>
	 *
	 *	@see com.zerog.ia.api.pub.CustomCodeAction#install
	 *	@see java.io.DataOutput
	 */
	public void install( InstallerProxy ip ) throws InstallException
	{
		if(isLoaded == true)
			return;
		isLoaded = true;


		/*	Get input from InstallAnywhere Variables.  The literal contents of
			the Variables are retieved into the Strings. */
		String source = ip.substitute( SOURCE_VAR_NAME );
		String destination = ip.substitute( DEST_VAR_NAME );



		/*	substitute() will return an empty string for any InstallAnywhere
			Variable that hasn't been assigned yet. */

		/*	If there is both a source and a destination, copy the files. */
		if (   source.equals("")
			|| destination.equals("") )
		{
			error( source, destination );
		}
		else
		{
			try {
				System.err.println(source + " -> " + destination);
				copyFile( source, destination );



				/*	Log the destination path to the uninstaller, so that later
					we may delete the file.*/
				DataOutput dout = ip.getLogOutput();

				dout.writeUTF( destination );

			} catch ( IOException ioe ) {
				throw new NonfatalInstallException( ioe.getMessage() );
			}
		}
	}

	/**
	 *	This is the method that is called at uninstall-time.  The DataInput
	 *	instance provides access to any information written at install-time
	 *	with the instance of DataOutput provided by UninstallerProxy.getLogOutput().<p>
	 *
	 *	For a bit more involved example, see InstallSharedDLL.
	 *
	 *	@see com.acme.installshareddll.InstallSharedDLL
	 *	@see com.acme.installshareddll.InstallSharedDLL#uninstall
	 *	@see com.zerog.ia.api.pub.CustomCodeAction#uninstall
	 *	@see java.io.DataInput
	 */
	public void uninstall( UninstallerProxy up ) throws InstallException
	{
		try {
			/*	The information written at install time to the DataOutput
				provided by InstallerProxy.getLogOutput() is now available from
				the uninstaller.  UninstallerProxy.getLogInput() returns an
				instance of java.io.DataInput.
				Be sure to read items in the same order they were written
				(with respect to the type).  For instance, if you wrote an
				int, some UTF and a double, read them back in the same order.*/
			DataInput din = up.getLogInput();
			String theDestinationPath = din.readUTF();
			File f = new File( theDestinationPath );
			f.delete();
		} catch ( IOException ioe ) {
			ioe.printStackTrace();
		}
	}

	/**
	 *	This method will be called to display a status message during the
	 *	installation.
	 *
	 *	@see com.zerog.ia.api.pub.CustomCodeAction#getInstallStatusMessage
	 */
	public String getInstallStatusMessage()
	{
		return INSTALL_MESSAGE;
	}

	/**
	 *	This method will be called to display a status message during the
	 *	uninstall.
	 *
	 *	@see com.zerog.ia.api.pub.CustomCodeAction#getUninstallStatusMessage
	 */
	public String getUninstallStatusMessage()
	{
		return UNINSTALL_MESSAGE;
	}

	public static void copyFile( String source, String destination )
		throws IOException
	{
		copyFile( new File( source ), new File( destination ) );
	}

	public static void copyFile( File source, String destination )
		throws IOException
	{
		copyFile( source, new File( destination ) );
	}

	public static void copyFile( String source, File destination )
		throws IOException
	{
		copyFile( new File( source ), destination );
	}

	/**
	 *	Copy the file represented by source to the file
	 *	represented by destination.  Makes use of OtherUtils.bufStreamCopy
	 *	to buffer the copy.
	 *
	 *	@see com.acme.fileutils.OtherUtils#bufStreamCopy
	 */
	public static void copyFile( File source, File destination )
		throws IOException
	{
		InputStream in = null;
		OutputStream out = null;

		try {

			in = new FileInputStream( source );
			out = new FileOutputStream( destination );

			OtherUtils.bufStreamCopy( in, out );

			out.flush();
			out.close();
			in.close();

		} finally {
			/*	If there's an IOException, we let it propagate to a higher
				level, but we try to close the files in any case. */
			if ( out != null )
			{
				out.flush();
				out.close();
			}

			if ( in != null )
			{
				in.close();
			}
		}
	}

	/**
	 *	Print something to indicate that the parameters were not acceptable.
	 */
	private void error( String source, String destination )
	{
		System.err.println( ERR_MSG );
		System.err.println( "Source: " + source );
		System.err.println( "Destination: " + destination );
	}
}
